#!/bin/sh

date_time=$(date +%m_%d_%y-%H:%M:%S)
echo "Started at" $date_time

data=data
script=script
bin=bin

mapper=$1
reads1=$2 
reads2=$3 
ref=$4 
resultfolder=$5

[ -d $resultfolder ] || mkdir $resultfolder

if [ "$mapper" = "bowtie2" ]
then

   # running Bowtie2 mapper on the read set and reference and making consensus from SAM file

   ./$script/bowtie2.sh $reads1 $reads2 $ref mapping_bt log_bt $resultfolder

   cd $resultfolder

   ../$bin/iuCount contigs_mapping_bt.fasta 

   iupacPercent=0
   filename="iu.txt"
   while read -r line
   do
   iupacPercent=$line
   done < "$filename"

   cd ..
 
   if [ $iupacPercent \< 0.01 ]
   then
      ./$script/EIMPipeline-bt.sh $reads1 $reads2 cns_mapping_bt.fa log_EIM $resultfolder	
   else
      ./$script/EIMPipeline-bt.sh $reads1 $reads2 ../$data/$ref log_EIM $resultfolder	
   fi
   
elif [ "$mapper" = "bwa" ]
then

   # running BWA mapper on the read set and reference and making consensus from SAM file

   ./$script/bwa.sh $reads1 $reads2 $ref mapping_bwa log_bwa $resultfolder


   # running EIM pipeline on the consensus of BWA using this mapper at the second step

   ./$script/EIMPipeline-bwa.sh $reads1 $reads2 cns_mapping_bwa.fa log_EIM $resultfolder

elif [ "$mapper" = "yara" ]
then

   # running Yara mapper on the read set and reference and making consensus from SAM file

   ./$script/yara.sh $reads1 $reads2 $ref mapping_yara log_yara $resultfolder


   # running EIM pipeline on the consensus of Yara using this mapper at the second step

   ./$script/EIMPipeline-yara.sh $reads1 $reads2 cns_mapping_yara.fa log_EIM $resultfolder

else
   echo "Error: The name of mapper is not correct."
fi

date_time=$(date +%m_%d_%y-%H:%M:%S)
echo "Finished at" $date_time
