#!/bin/sh

# ------------ EIM pipeline ---------------

date_time=$(date +%m_%d_%y-%H:%M:%S)

data=data
src=bin
script=script

#mkdir $result

reads1=$1 
reads2=$2 
cns_bt=$3 
log=$4
result=$5

###################
cd $result
###################

exec 1>>$log
exec 2>>$log

echo "//###########################################"
echo "The pipeline started at" $date_time 
echo "//###########################################"

echo "./$src/preProcess $cns_bt ref_n.fa ../$data/$reads1 ../$data/$reads2 $result"
../$src/preProcess $cns_bt ref_n.fa ../$data/$reads1 ../$data/$reads2 $result

cns_bt=ref_n.fa

echo "../$src/ctgCount contigs_mapping_bt.fasta"
../$src/ctgCount contigs_mapping_bt.fasta

echo "bowtie2-build --quiet $cns_bt ref_exact"
bowtie2-build --quiet $cns_bt ref_exact

echo "bowtie2 --time --score-min 'C,0,-1' -x ref_exact -1 ../$data/$reads1 -2 ../$data/$reads2 -S exactMapping_bt.sam"
bowtie2 --time --score-min 'C,0,-1' -x ref_exact -1 ../$data/$reads1 -2 ../$data/$reads2 -S exactMapping_bt.sam

echo "//###########################################\n"

echo "separateUAM.pl exactMapping_bt.sam"
perl ../$script/separateUAM.pl exactMapping_bt.sam

echo "extractReads.pl $reads1 $reads2 UAM_exactMapping_bt.sam"
perl ../$script/extractReads.pl ../$data/$reads1 ../$data/$reads2 UAM_exactMapping_bt.sam
echo "//###########################################\n"

echo "samtools faidx $cns_bt"
samtools faidx $cns_bt

echo "samtools import ${cns_bt}.fai exactMapping_bt.sam exactMapping_bt.bam"
samtools import ${cns_bt}.fai exactMapping_bt.sam exactMapping_bt.bam

echo "samtools sort exactMapping_bt.bam -o exactMapping_bt.sort.bam"
samtools sort exactMapping_bt.bam -o exactMapping_bt.sort.bam

echo "samtools index exactMapping_bt.sort.bam"
samtools index exactMapping_bt.sort.bam

echo "samtools mpileup --min-MQ 42 -A -uf $cns_bt exactMapping_bt.sort.bam | bcftools call -c > exactMapping_bt.vcf"
samtools mpileup --min-MQ 42 -A -uf $cns_bt exactMapping_bt.sort.bam | bcftools call -c > exactMapping_bt.vcf

echo "perl ../$script/convertVcf2Cns.pl exactMapping_bt.vcf cns_exact_bt.fa"
perl ../$script/convertVcf2Cns.pl exactMapping_bt.vcf cns_exact_bt.fa

echo "../$src/convertCnsToCntgs cns_exact_bt.fa contigs_exact_bt.fasta"
../$src/convertCnsToCntgs cns_exact_bt.fa contigs_exact_bt.fasta
echo "//###########################################\n"

echo "../$src/makeSupContigs cns_exact_bt.fa $cns_bt contigs_sup.fasta"
../$src/makeSupContigs cns_exact_bt.fa $cns_bt contigs_sup.fasta

echo "bowtie2-build --quiet contigs_sup.fasta contigs_sup"
bowtie2-build --quiet contigs_sup.fasta contigs_sup

echo "bowtie2 --time -x contigs_sup -1 UAM_${reads1} -2 UAM_${reads2} -S UAM_contigs_sup_bt.sam"
bowtie2 --time -x contigs_sup -1 UAM_${reads1} -2 UAM_${reads2} -S UAM_contigs_sup_bt.sam
echo "//###########################################\n"

echo "samtools faidx contigs_sup.fasta"
samtools faidx contigs_sup.fasta

echo "samtools import contigs_sup.fasta.fai UAM_contigs_sup_bt.sam UAM_contigs_sup_bt.bam"
samtools import contigs_sup.fasta.fai UAM_contigs_sup_bt.sam UAM_contigs_sup_bt.bam

echo "samtools sort UAM_contigs_sup_bt.bam -o UAM_contigs_sup_bt.sort.bam"
samtools sort UAM_contigs_sup_bt.bam -o UAM_contigs_sup_bt.sort.bam

echo "samtools index UAM_contigs_sup_bt.sort.bam"
samtools index UAM_contigs_sup_bt.sort.bam

echo "samtools mpileup -A -uf contigs_sup.fasta UAM_contigs_sup_bt.sort.bam | bcftools call --keep-masked-ref -c > UAM_contigs_sup_bt.vcf"
samtools mpileup -A -uf contigs_sup.fasta UAM_contigs_sup_bt.sort.bam | bcftools call --keep-masked-ref -c > UAM_contigs_sup_bt.vcf

echo "perl ../$script/convertVcf2Cns3.pl UAM_contigs_sup_bt.vcf cns_UAM_contigs_sup_bt.fa"
perl ../$script/convertVcf2Cns3.pl UAM_contigs_sup_bt.vcf cns_UAM_contigs_sup_bt.fa

echo "//###########################################\n"

echo "../$src/merge_exact_UAM contigs_exact_bt.fasta cns_UAM_contigs_sup_bt.fa cns_EIM_bt.fa"
../$src/merge_exact_UAM contigs_exact_bt.fasta cns_UAM_contigs_sup_bt.fa cns_EIM_bt.fa

echo "cat cns_EIM_bt.fa | seqtk cutN -n 1 - | seqtk seq -L 500 - > contigs_EIM.fasta"
cat cns_EIM_bt.fa | seqtk cutN -n 1 - | seqtk seq -L 500 - > contigs_EIM.fasta

date_time=$(date +%m_%d_%y-%H:%M:%S)
echo "//###########################################"
echo "The pipeline finished at" $date_time
