#!/bin/sh

# ------------ EIM pipeline ---------------

date_time=$(date +%m_%d_%y-%H:%M:%S)

data=data
src=bin
script=script

#mkdir $result

reads1=$1 
reads2=$2 
cns_yara=$3 
log=$4
result=$5

###################
cd $result
###################

exec 1>>$log
exec 2>>$log

echo "//###########################################"
echo "The pipeline started at" $date_time 
echo "//###########################################"

echo "../$src/ctgCount contigs_mapping_bt.fasta"
../$src/ctgCount contigs_mapping_bt.fasta

echo "../$src/SBA_indexer $cns_yara ref_exact 0 0"
../$src/SBA_indexer $cns_yara ref_exact 0 0

echo "../$src/SBA_exactMapper ../$data/$reads1 ../$data/$reads2 250 250 ref_exact exactMapping_sba.sam 1"
../$src/SBA_exactMapper ../$data/$reads1 ../$data/$reads2 250 250 ref_exact exactMapping_sba.sam 1
echo "//###########################################\n"

echo "../$src/makeEUConsensus exactMapping_sba.sam cns_exact_sba.fa"
../$src/makeEUConsensus exactMapping_sba.sam cns_exact_sba.fa

echo "../$src/convertCnsToCntgs cns_exactMapping_sba.fa contigs_exact_sba.fasta"
../$src/convertCnsToCntgs cns_exact_sba.fa contigs_exact_sba.fasta
echo "//###########################################\n"

echo "makeSupContigs cns_exactMapping_yara.fa $cns_yara contigs_sup.fasta"
../$src/makeSupContigs cns_exact_sba.fa $cns_yara contigs_sup.fasta

echo "yara_indexer contigs_sup.fasta contigs_sup"
yara_indexer contigs_sup.fasta -o contigs_sup

echo "yara_mapper  -v --secondary-alignments omit -ll 250 -ld 250 contigs_sup UAM_${reads1}  UAM_${reads2} -o UAM_contigs_sup_yara.sam"
yara_mapper  -v --secondary-alignments omit -ll 250 -ld 250 contigs_sup UAM_${reads1}  UAM_${reads2} -o UAM_contigs_sup_yara.sam
echo "//###########################################\n"

echo "samtools faidx contigs_sup.fasta"
samtools faidx contigs_sup.fasta

echo "samtools import contigs_sup.fasta.fai UA_contigs_sup_yara.sam UA_contigs_sup_yara.bam"
samtools import contigs_sup.fasta.fai UAM_contigs_sup_yara.sam UAM_contigs_sup_yara.bam

echo "samtools sort UA_contigs_sup_yara.bam -o UA_contigs_sup_yara.sort.bam"
samtools sort UAM_contigs_sup_yara.bam -o UAM_contigs_sup_yara.sort.bam

echo "samtools index UA_contigs_sup_yara.sort.bam"
samtools index UAM_contigs_sup_yara.sort.bam

echo "samtools mpileup -A -uf contigs_sup.fasta UA_contigs_sup_yara.sort.bam | bcftools call --keep-masked-ref -c > UA_contigs_sup_yara.vcf"
samtools mpileup -A -uf contigs_sup.fasta UAM_contigs_sup_yara.sort.bam | bcftools call --keep-masked-ref -c > UAM_contigs_sup_yara.vcf

echo "perl ../$script/convertVcf2Cns3.pl UA_contigs_sup_yara.vcf cns_UA_contigs_sup_yara.fa"
perl ../$script/convertVcf2Cns3.pl UAM_contigs_sup_yara.vcf cns_UAM_contigs_sup_yara.fa

echo "//###########################################\n"

echo "../$src/merge_exact_UAM contigs_exactMapping_sba.fasta cns_UA_contigs_sup_yara.fa cns_EIM_sba_yara.fa"
../$src/merge_exact_UAM contigs_exact_sba.fasta cns_UAM_contigs_sup_yara.fa cns_EIM_sba_yara.fa

echo "cat cns_EIM_sba_yara.fa | seqtk cutN -n 1 - | seqtk seq -L 500 - > contigs_EIM_sba_yara.fasta"
cat cns_EIM_sba_yara.fa | seqtk cutN -n 1 - | seqtk seq -L 500 - > contigs_EIM.fasta

date_time=$(date +%m_%d_%y-%H:%M:%S)
echo "//###########################################"
echo "The pipeline finished at" $date_time
