#!/bin/sh

# ------------ Bowtie2 ---------------

date_time=$(date +%m_%d_%y-%H:%M:%S)

data=data
src=src
script=script

#mkdir $result

reads1=$1 
reads2=$2 
ref=$3 
Mapping_bt=$4 
log=$5
result=$6 

###################
cd $result
###################

exec 1>>$log
exec 2>>$log

echo "//###########################################"
echo "Started at" $date_time 
echo "//###########################################"

echo "bowtie2-build --quiet ../$data/$ref ref"
bowtie2-build --quiet ../$data/$ref ref

echo "bowtie2 --threads 10 --time -x ref -1 ../$data/$reads1 -2 ../$data/$reads2 -S $Mapping_bt.sam"
bowtie2 --time --threads 10 -x ref -1 ../$data/$reads1 -2 ../$data/$reads2 -S $Mapping_bt.sam

echo "//###########################################\n"

echo "samtools faidx ../$data/$ref"
samtools faidx ../$data/$ref

echo "samtools import ${ref}.fai $Mapping_bt.sam $Mapping_bt.bam"
samtools import ${ref}.fai $Mapping_bt.sam $Mapping_bt.bam

echo "samtools sort $Mapping_bt.bam -o $Mapping_bt.sort.bam"
samtools sort $Mapping_bt.bam -o $Mapping_bt.sort.bam

echo "samtools index $Mapping_bt.sort.bam"
samtools index $Mapping_bt.sort.bam

echo "samtools mpileup -uf ../$data/$ref $Mapping_bt.sort.bam | bcftools call -keep-masked-ref -c > $Mapping_bt.vcf"
samtools mpileup -uf ../$data/$ref $Mapping_bt.sort.bam | bcftools call --keep-masked-ref -c > $Mapping_bt.vcf

echo "perl ../$script/convertVcf2Cns.pl $Mapping_bt.vcf cns_$Mapping_bt.mc.fa readLen"
perl ../$script/convertVcf2Cns.pl $Mapping_bt.vcf cns_$Mapping_bt.fa

echo "cat cns_$Mapping_bt.mc.fa | seqtk cutN -n 1 - | seqtk seq -L 10 - > contigs_$Mapping_bt.mc.fasta"
cat cns_$Mapping_bt.fa | seqtk cutN -n 1 - | seqtk seq -L 10 - > contigs_$Mapping_bt.fasta

date_time=$(date +%m_%d_%y-%H:%M:%S)
echo "//###########################################"
echo "Finished at" $date_time

[ -e $Mapping_bt.sam ] && rm $Mapping_bt.sam

[ -e $Mapping_bt.sort.bam ] && rm $Mapping_bt.sort.bam

[ -e $Mapping_bt.vcf ] && rm $Mapping_bt.vcf

[ -e $Mapping_bt_Temp.vcf ] && rm $Mapping_bt_Temp.vcf



