#!/bin/sh

# ------------ Bowtie2 ---------------

date_time=$(date +%m_%d_%y-%H:%M:%S)

data=data
src=src
script=script

#mkdir $result

reads1=$1 
reads2=$2 
ref=$3 
Mapping_bwa=$4 
log=$5
result=$6 

###################
cd $result
###################

exec 1>>$log
exec 2>>$log

echo "//###########################################"
echo "Started at" $date_time 
echo "//###########################################"

echo "bwa index ../$data/$ref -p ref"
bwa index ../$data/$ref -p ref

echo "bwa aln ref ../$data/$reads1 > ${Mapping_bwa}_1.sai"
bwa aln ref ../$data/$reads1 > ${Mapping_bwa}_1.sai

echo "bwa aln ref ../$data/$reads2 > ${Mapping_bwa}_2.sai"
bwa aln ref ../$data/$reads2 > ${Mapping_bwa}_2.sai

echo "bwa sampe ref ${Mapping_bwa}_1.sai ${Mapping_bwa}_2.sai ../$data/$reads1  ../$data/$reads2 > $Mapping_bwa.sam"
bwa sampe ref ${Mapping_bwa}_1.sai ${Mapping_bwa}_2.sai ../$data/$reads1  ../$data/$reads2 > $Mapping_bwa.sam

date_time=$(date +%m_%d_%y-%H:%M:%S)
echo "//###########################################"
echo "End of Mapping" $date_time

echo "samtools faidx ../$data/$ref"
samtools faidx ../$data/$ref

echo "samtools import ${ref}.fai $Mapping_bwa.sam $Mapping_bwa.bam"
samtools import ${ref}.fai $Mapping_bwa.sam $Mapping_bwa.bam

echo "samtools sort $Mapping_bwa.bam -o $Mapping_bwa.sort.bam"
samtools sort $Mapping_bwa.bam -o $Mapping_bwa.sort.bam

echo "samtools index $Mapping_bwa.sort.bam"
samtools index $Mapping_bwa.sort.bam

echo "samtools mpileup -uf ../$data/$ref $Mapping_bwa.sort.bam | bcftools call -keep-masked-ref -c > $Mapping_bwa.vcf"
samtools mpileup -uf ../$data/$ref $Mapping_bwa.sort.bam | bcftools call --keep-masked-ref -c > $Mapping_bwa.vcf

echo "perl ../$script/convertVcf2Cns.pl $Mapping_bwa.vcf cns_$Mapping_bwa.fa"
perl ../$script/convertVcf2Cns.pl $Mapping_bwa.vcf cns_$Mapping_bwa.fa

echo "cat cns_$Mapping_bwa.fa | seqtk cutN -n 1 - | seqtk seq -L 10 - > contigs_$Mapping_bwa.fasta"
cat cns_$Mapping_bwa.fa | seqtk cutN -n 1 - | seqtk seq -L 10 - > contigs_$Mapping_bwa.fasta

date_time=$(date +%m_%d_%y-%H:%M:%S)
echo "//###########################################"
echo "Finished at" $date_time

[ -e $Mapping_bwa.sam ] && rm $Mapping_bwa.sam

[ -e $Mapping_bwa.sort.bam ] && rm $Mapping_bwa.sort.bam

[ -e $Mapping_bwa.vcf ] && rm $Mapping_bwa.vcf

[ -e $Mapping_bwa_Temp.vcf ] && rm $Mapping_bwa_Temp.vcf
