#!/bin/sh

# ------------ Bowtie2 ---------------

date_time=$(date +%m_%d_%y-%H:%M:%S)

data=data
src=src
script=script

#mkdir $result

reads1=$1 
reads2=$2 
ref=$3 
Mapping_yara=$4 
log=$5
result=$6 

###################
cd $result
###################

exec 1>>$log
exec 2>>$log

echo "//###########################################"
echo "Started at" $date_time 
echo "//###########################################"

echo "yara_indexer ../$data/$ref -o ref.index"
yara_indexer ../$data/$ref -o ref.index

echo "yara_mapper  -v --secondary-alignments omit -ll 250 -ld 250 ref.index  ../$data/$reads1  ../$data/$reads2 -o $Mapping_yara.sam"
yara_mapper  -v --secondary-alignments omit -ll 250 -ld 250 ref.index  ../$data/$reads1  ../$data/$reads2 -o $Mapping_yara.sam
echo "//###########################################\n"

echo "samtools faidx ../$data/$ref"
samtools faidx ../$data/$ref

echo "samtools import ${ref}.fai $Mapping_yara.sam $Mapping_yara.bam"
samtools import ${ref}.fai $Mapping_yara.sam $Mapping_yara.bam

echo "samtools sort $Mapping_yara.bam -o $Mapping_yara.sort.bam"
samtools sort $Mapping_yara.bam -o $Mapping_yara.sort.bam

echo "samtools index $Mapping_yara.sort.bam"
samtools index $Mapping_yara.sort.bam

echo "samtools mpileup -uf ../$data/$ref $Mapping_yara.sort.bam | bcftools call -keep-masked-ref -c > $Mapping_yara.vcf"
samtools mpileup -uf ../$data/$ref $Mapping_yara.sort.bam | bcftools call --keep-masked-ref -c > $Mapping_yara.vcf

echo "perl ../$script/convertVcf2Cns.pl $Mapping_yara.vcf cns_$Mapping_yara.fa"
perl ../$script/convertVcf2Cns.pl $Mapping_yara.vcf cns_$Mapping_yara.fa

echo "cat cns_$Mapping_yara.fa | seqtk cutN -n 1 - | seqtk seq -L 10 - > contigs_$Mapping_yara.fasta"
cat cns_$Mapping_yara.fa | seqtk cutN -n 1 - | seqtk seq -L 10 - > contigs_$Mapping_yara.fasta

date_time=$(date +%m_%d_%y-%H:%M:%S)
echo "//###########################################"
echo "Finished at" $date_time

[ -e $Mapping_yara.sam ] && rm $Mapping_yara.sam

[ -e $Mapping_yara.sort.bam ] && rm $Mapping_yara.sort.bam

[ -e $Mapping_yara.vcf ] && rm $Mapping_yara.vcf

[ -e $Mapping_yara_Temp.vcf ] && rm $Mapping_yara_Temp.vcf
