/////////////////////////////////////////////
// This program Aligns sequenced reads to 
// a reference sequence by 
// indexing the referece using a hash table 
// and searching each read in the hash table
//--------------------------
// Types of checking: Exact, one Mutation, 
// one Insersion
// -------------------------
// For each read, it checkes the all types 
// and then outputs the first answer
/////////////////////////////////////////////

#include <iostream>
#include <fstream>
#include <string>
#include <unordered_map>
#include <stdlib.h>
#include <ctime>
#include <math.h>

using namespace std;

//////////////// CONSTANTS ////////////////
/*
const int kitrate = 3;

const int A = 3;
const int C = 20;
const int G = 61;
const int T = 98;

const int A2 = 9;
const int C2 = 400;
const int G2 = 3721;
const int T2 = 9604;

const int ScM[4] = {3,20,61,98};
*/


const int kitrate = 2;

const int A = 1;
const int C = 2;
const int G = 5;
const int T = 7;

const int A2 = 5;
const int C2 = 1;
const int G2 = 7;
const int T2 = 2;

const int ScM[4] = {1,2,5,7};

/*
const int kitrate = 3;

const int A = 1;
const int C = 2;
const int G = 3;
const int T = 5;

const int A2 = 3;
const int C2 = 1;
const int G2 = 5;
const int T2 = 2;

const int ScM[4] = {1,2,3,5};
*/

const char ScC[4] = {'A','C','G','T'};

const string WHITESPACE = " \t\f\v\n\r";

const int REPEAT_NUM = 20; 

const int MAX_NUM = 1000;

///////////////////////////////////////////

///////////// GLOBAL VARIABLES ////////////

// INDEX OF GENOME
unordered_multimap<string, string> Index;
// REAL POSITIONS OF READS AT GENOME 
string *GenomePos;

// SHORT READ INDEX
int Sh_Ind = 0;
// NUMBER OF UNALIGNED READS
int u = 0;       
// NUMBER OF READS ALIGNED JUST ONCE
int Once = 0;
// NUMBER OF READS ALIGNED MORE THAN ONCE
int r = 0;
// NUMBER OF READS ALIGNED AT THE CORRECT POSITION
//int correctAligned = 0;
// NUMBER OF READS WERE RECONIZED PAIR
int paired = 0;
// MULTI READS ADD TO UNALIGNED READS
int multiAddToUnAligned;

//bool found;

/*
//mean: library insert average length
int dist = 150; 
// stdev: library insert standard deivation length
int sd = 10 * 6; //10 
// read Length
int rLen = 45; 
*/


//mean: library insert average length
int dist; 
// stdev: library insert standard deivation length
int sd; 
// read Length
int rLen; 

/*
//mean: library insert average length
int dist = 155; 
// stdev: library insert standard deivation length
int sd = 100; //10 
// read Length
int rLen; 
*/
////////////////////////////////////////

//////////////// METHODS ///////////////

int readingGenome(char *IndexFile, long[], string[]);
void readingIndex(char *);
void mappingReads (char *, char *, char *, char *);
void printResults();

string tryExact (string, string, char);

bool isPaired (string&, string&, int);

int NucScore(char);
int NucScore2(char);
inline string trim (const string &);
string reverseComplement (string);
string reverse (string);
int countUniquePos (string &);
void initBF (int [], bool);
char complement (char);

////////////////////////////////////////

int main(int argc, char *argv[])
{
	if (argc < 8) 
	{
		cerr << "Usage: " << argv[0] <<" "
		     << "ReadFile1 ReadFile2 dist sd IndexFile ResultFile multiAddToUnAligned\n";    
		return 1;
	}
	
	char *ReadFile1 = argv[1];
	char *ReadFile2 = argv[2];
	dist = stoi (argv[3]); 
	sd = stoi (argv[4]); 
	char *IndexFile = argv[5];
	char *ResultFile = argv[6];
	multiAddToUnAligned = stoi (argv[7]);
	
	time_t now = time(0);
	char* dt = ctime(&now);
	cout << "Started at " << dt << "\n";
	
	readingIndex(IndexFile);

	now = time(0);
	dt = ctime(&now);
	cout << "Finished Reading Index File at " << dt << "\n";
	
	mappingReads (ReadFile1, ReadFile2, ResultFile, IndexFile);
	
	printResults();

	now = time(0);
	dt = ctime(&now);
	cout << "Finished at " << dt << "\n";
	
	return 0;
}
/////////////////////////////////////////////////////

///////////////////// FUNCTIONS /////////////////////

int readingGenome(char *IndexFile, long CtgLen[], string CtgName[]) //*** Totally has been changed
//-----------------------------------------------------------------
{
	string CtgNamesFile(IndexFile);
	CtgNamesFile.append("_name");

	ifstream inf(CtgNamesFile.c_str());
	if (!inf)
	{
		cerr << CtgNamesFile <<" could not be opened for reading!\n";
	}
	
	int i = 0; 
	int i_space;
	string line;

	while (!inf.eof())
	{
		getline(inf, line);
		
		if (line.length() == 0) // for the last empty line
			break;

		i_space = -1;
		while (line[++i_space] != '\t');
		
		CtgName[i] = line.substr(0, i_space);
		CtgLen[i] = stol (line.substr(i_space + 1, line.length() - i_space));

		i++;
	}
	
	return i;
} 


void readingIndex(char *IndexFile)  //*** without any changes
//--------------------------------
{
	ifstream inIndexf(IndexFile);
	if (!inIndexf)
	{
		cerr << IndexFile <<" could not be opened for reading!\n";
		return;
	}
	
	string line;
	int i_space;
	string key;
	string value = "0";
	
	while (!inIndexf.eof())
	{
		//????????????????
		//if (stoi(value) %1000000 == 0)
			//cout << value << "\n";
			
		getline(inIndexf, line);
		
		if (line.length() == 0) // for the last empty line
			break;
			
		i_space = -1;
		while (line[++i_space] != '\t');
		
		key = line.substr(i_space + 1, line.length() - i_space);
		value = line.substr(0, i_space);
		
		Index.insert(pair<string, string>(key, value));	
	}
	
	//auto range = Index.equal_range("53040797.3920990363");
	
	//for (auto it = range.first;  it != range.second; ++it) 
	//{
	//	cout << it->second << "\n";	
	//}
}

void mappingReads (char *ReadFile1, char *ReadFile2, char *ResultFile, char *IndexFile)  //*** with some changes
//------------------------------------------------------------------------------------
{
	ofstream outf(ResultFile);
	if (!outf)
	{
		cerr << ResultFile <<" could not be opened for writing!\n";
		return;
	}
	
	string RF(ReadFile1);
	RF = RF.substr (RF.find_last_of ("/\\") + 1);

	string UA_ReadFile1;
	if (multiAddToUnAligned)
		UA_ReadFile1 = "UAM_";
	else
		UA_ReadFile1 = "UA_";

	UA_ReadFile1.append(RF);
	ofstream outf2(UA_ReadFile1.c_str());
	if (!outf2)
	{
		cerr << UA_ReadFile1 <<" could not be opened for writing!\n";
		return;
	}

	string RF2(ReadFile2);
	RF2 = RF2.substr (RF2.find_last_of ("/\\") + 1);

	string UA_ReadFile2;
	if (multiAddToUnAligned)
		UA_ReadFile2 = "UAM_";
	else
		UA_ReadFile2 = "UA_";	

	
	UA_ReadFile2.append(RF2);
	ofstream outf3(UA_ReadFile2.c_str());
	if (!outf3)
	{
		cerr << UA_ReadFile2 <<" could not be opened for writing!\n";
		return;
	}

///////////////  CHANGE  //////////////////

	long   CtgLen  [MAX_NUM];
	string CtgName [MAX_NUM];
	int ctgNum;

	ctgNum = readingGenome (IndexFile, CtgLen, CtgName);
	
	///////////// WRITING DOWN HEADINGS OF SAM FILE /////////////
	
	outf << "@HD\tVN:1.0\tSO:unsorted\n";

	for (int i = 0; i < ctgNum; i++)
	{
		outf << "@SQ\tSN:"<< CtgName[i] <<"\tLN:"<< CtgLen[i] <<"\n";
	}

	outf << "@PG\tID:SBA\tPN:SBA\tVN:1.0.0\n";

///////////////  CHANGE  //////////////////

	string line1 = "";
	string line2;
	string line3;
	string line4;
	
	string pre_line1;
	string pre_line2;
	string pre_line3;
	string pre_line4;
	
	int dist;	
	int ReadLength;
	
	bool flag, pre_flag;
	
	char var, pre_var;
	int varPos, pre_varPos;
	int vp;
	char varNuc, pre_varNuc;
	char vn;
	
	// SAMFILE VARAIBLES
	string pos, pre_pos;
	string cigar, pre_cigar;
	
	int bf, pre_bf;
	string RNext;
	
	int pre_f[16], f[16];
	/////////////////////////////
	initBF (pre_f, true);
	initBF (f, false);
	/////////////////////////////
	ifstream inReadf1(ReadFile1);
	if (!inReadf1)
	{
		cerr << ReadFile1 <<" could not be opened for reading!\n";
		return;
	}

	ifstream inReadf2(ReadFile2);
	if (!inReadf2)
	{
		cerr << ReadFile2 <<" could not be opened for reading!\n";
		return;
	}

	int kk = 1;
//	int cc = 0;
	string rline2;
	
	while(!inReadf1.eof()) 
	{
		//cout << cc++ << "\n";//????????///////
		
		if (line1 != "")
		{
			pre_flag = flag;
			pre_line1 = line1;
			pre_line2 = line2;
			pre_line3 = line3;
			pre_line4 = line4;
		
			pre_pos = pos;
			pre_bf = bf;
			pre_cigar = cigar;
		}
		
		if (kk % 2 == 1)
		{
			// LINE1: TITLE OF SHORT READ
			getline(inReadf1, line1);
		
			// LINE2: CONTENT OF SHORT READ
			getline(inReadf1, line2);
		
			// LINE3: DESCRIPTION OF SHORT READ
			getline(inReadf1, line3);
		
			// LINE4: QUALITY OF SHORT READ
			getline(inReadf1, line4);

			///////////////////////////
			initBF (pre_f, true);
			initBF (f, false);
			///////////////////////////
		} 
		else
		{
			// LINE1: TITLE OF SHORT READ
			getline(inReadf2, line1);
		
			// LINE2: CONTENT OF SHORT READ
			getline(inReadf2, line2);
		
			// LINE3: DESCRIPTION OF SHORT READ
			getline(inReadf2, line3);
		
			// LINE4: QUALITY OF SHORT READ
			getline(inReadf2, line4);
		}
		
		if (line1 == "")
			break;
		/*	
		int i_slash = line1.find ("/");
		if (i_slash != string::npos)
		{
			line1 = line1.substr (1, i_slash - 1);	
		}*/
			
		line1 = line1.substr (1, line1.length() - 1);
					
		Sh_Ind++;
		
		
		flag = false;
		
		//=====================================
		string tt = ""; 
		//bf = 0;		
		pos = tryExact (line1, line2, 0);
		rline2 = reverseComplement (line2);
		tt = tryExact (line1, rline2, 1);
		if (pos == "")
		{
			pos = tt;
		}
		else 
		{
			if ( tt != "")
				pos = pos + "," + tt;
		}
		
		//=====================================
		
		
		if (pos != "")
		{
			flag = true;			
			cigar = to_string(line2.length()) + "M";

			if (kk % 2 == 1)
			{
				pre_f[2] = 0; // hex: 0x0004 The query sequence itself was mapped
				f[3] = 0;
			}
			else
			{
				f[2] = 0;
				pre_f[3] = 0; // hex: 0x0008 The mate was mapped	
			}	
		}
		
		if (!flag)
		{
			cigar = "*";
			u++;
		}
		
		if (kk % 2 == 0 && (!flag || !pre_flag))
		{
			outf2 << "@" << pre_line1 << "\n" << pre_line2 << "\n" << pre_line3 << "\n" << pre_line4 <<"\n";
			outf3 << "@" << line1 << "\n" << line2 << "\n" << line3 << "\n" << line4 <<"\n";
		}
		

		///////////CHECKING POSITIONS BY PAIRING INFO////////////
		
		if (kk % 2 == 0 && (flag || pre_flag))
		{
			if (isPaired (pre_pos, pos, line2.length()))
			{
				paired++;
				pre_f[1] = 1; // hex: 0x0002
				f[1] = 1; // hex: 0x0002
				
			}
			
			long POS1 = -1; //stoi (pre_pos, nullptr);
			bool RC1;
			string PNEXT1 = "0";
			string RNAME1;  //***
			string SEQ1 = pre_line2;
			string QUAL1 = pre_line4;
			
			long POS2 = -1; //stoi (pos, nullptr);
			bool RC2;
			string PNEXT2 = "0";
			string RNAME2;  //***
			string SEQ2 = line2;
			string QUAL2 = line4;

			string RNEXT = "=";
			int TLEN = 0;

			bool pre_unique = true;
			bool unique = true;

			//////////////////////////////////////////////////
			
			RC1 = false;
			if (pre_flag)
			{
				if (countUniquePos(pre_pos) == 1)
				{
					if (pre_pos.substr (0,1) == "r")
					{
						pre_pos = pre_pos.substr (1, pre_pos.length() - 1);
						RC1 = true;
					}

///////////////  CHANGE  //////////////////

					int i_dot, rNum;
					i_dot = pre_pos.find (".");	
					rNum = stoi (pre_pos.substr (0, i_dot));
					POS1 = stol (pre_pos.substr(i_dot + 1, pre_pos.length() - i_dot));	
					RNAME1 = CtgName[rNum];
					//cout << "1:" << pre_pos << " -- " << POS1 << " --- " << RNAME1 << "\n";

///////////////  CHANGE  //////////////////

					Once++;
				}
				else
				{
					pre_unique = false;
					r++;
				}
			}
			
			RC2 = false;
			if (flag)
			{
				if (countUniquePos(pos) == 1)
				{
					if (pos.substr (0,1) == "r")
					{
						pos = pos.substr (1, pos.length() - 1);
						RC2 = true;
					}
///////////////  CHANGE  //////////////////

					int i_dot, rNum;
					i_dot = pos.find (".");	
					rNum = stoi (pos.substr (0, i_dot));
					POS2 = stol (pos.substr(i_dot + 1, pos.length() - i_dot));	
					RNAME2 = CtgName[rNum];
					//cout << "2:"<< pos << " -- " << POS2 << " --- " << RNAME2 << "\n";
					//char ch;
					//cin >> ch;

///////////////  CHANGE  //////////////////
					Once++;
				}
				else
				{
					unique = false;
					r++;
				}
	          	}
	          	/////////////////// BF && PRE_BF ///////////////////
			int PRE_BF;
	          	int BF;
			
          		if (RC1)
          		{
          			SEQ1 = reverseComplement (pre_line2);
				QUAL1 = reverse (pre_line4);
				pre_f[4] = 1; // hex: 0x0010 strand of query is reverse 
				f[5] = 1;
          		}
          		else 
          		{
          			SEQ1 = pre_line2;
				QUAL1 = pre_line4;	
          		}
          	
          		if (RC2)
          		{
          			SEQ2 = reverseComplement (line2);
				QUAL2 = reverse (line4);
				pre_f[5] = 1; // hex: 0x0020 strand of mate is reverse
				f[4] = 1; 
				
          		}
          		else
          		{
          			SEQ2 = line2;
				QUAL2 = line4;
			}
          	
          		PRE_BF = 0;
          		BF = 0;
		    	for (int j = 0; j < 16; j++)
			{
				PRE_BF += pow (2, j)*pre_f[j];
				BF += pow (2, j)*f[j];
			}
			///////////////////////////////////////////////////////
			
			dist = POS2 - POS1;   
			
			if (dist > 0)
				dist += line2.length();
			else
				dist -= line2.length();
			
			//if (RNAME1.compare(RNAME2) == 0)	//*** if two contigs are equal
			TLEN =  dist; 
			
			///////////////////////////////////////////////////////
			
			if (pre_flag && pre_unique)
			{ 	
				string readName = pre_line1;
				size_t found;
				found = readName.find(" "); //WHITESPACE
				if (found != string::npos)
					readName.erase(found + 1);
				//*** 	
				outf << readName << "\t"<< PRE_BF << "\t"<< RNAME1 << "\t" 
				     << POS1 <<"\t"<< 42 << "\t" << pre_cigar <<"\t"<< RNEXT <<"\t" // 42-->255 
				     << POS2 <<"\t"<< TLEN << "\t"						    
				     << SEQ1 <<"\t" << QUAL1 << "\t" << "NM:i:1"<<"\n";
			}	
			
			if (flag && unique)
			{	
				string readName = line1;
				size_t found;
				found = readName.find(" ");
				if (found != string::npos)
					readName.erase(found + 1);
				//*** 	
				outf << readName << "\t"<< BF << "\t"<< RNAME2 <<"\t" 
			             << POS2 <<"\t"<< 42 << "\t" << cigar <<"\t"<< RNEXT <<"\t" 
				     << POS1 << "\t" << -TLEN << "\t"
				     << SEQ2 <<"\t"<< QUAL2 << "\t" << "NM:i:1" <<"\n";
	         	}


			if (multiAddToUnAligned && pre_flag && flag && (!pre_unique || !unique))
			{
				outf2 << "@" << pre_line1 << "\n" << pre_line2 << "\n" << pre_line3 << "\n" << pre_line4 <<"\n";
				outf3 << "@" << line1 << "\n" << line2 << "\n" << line3 << "\n" << line4 <<"\n";	
			} 
			//////////////////////////////////////////////////
			
		}
		
		kk++;
	}
}
void printResults()
//-----------------
{
	Index.clear();
	
	double up = ((double)u / (double)Sh_Ind) * 100;
	double op = ((double)Once / (double)Sh_Ind) * 100;
	double rp = ((double)r / (double)Sh_Ind) * 100;
	double Overallp = ((double)(Once + r) / (double)Sh_Ind) * 100;
//	double ca = (double)correctAligned / (double)Sh_Ind * 100;

	cout << Sh_Ind << " reads\n";
	cout << "aligned 0 times: "<< u <<"\t" << up <<" %\n";
	cout << "aligned exactly 1 times: " << Once << "\t"<< op <<" %\n";
	cout << "aligned >1 times: " << r << "\t"<< rp <<" %\n";
	cout << Overallp <<" % overall alignment rate\n";
//	cout << correctAligned <<"  correct overall alignment\n"; //------------test for debuging
//	cout << ca <<"  %correct overall alignment\n";
	cout << paired*2 <<" paired found\n";
}


string tryExact (string line1, string line2, char rev)  // *** without any chages
//---------------------------------------------------
{	
	int ReadLength = line2.length();
		
	long Score = 0;
	unsigned long long Score2 = 0;
	int i , j, k = 0;
	for (j = 0; j < ReadLength; j++)
	{
		k = pow ((j + 1), kitrate);
		Score += (k * NucScore(line2[j]));
		Score2 += (k * NucScore2(line2[j])); 
	}
	
	string key = to_string(Score) + "." + to_string(Score2);
	auto range = Index.equal_range(key);
	string Pos = "";
	string temp = "";
	int PosCount = 0;
	
	for (auto it = range.first;  it != range.second; ++it) 
	{
		temp = it->second;
		
		if (rev == 1)
			temp = "r"+temp;
		Pos += temp;
		Pos +=",";
		PosCount++;		
	}
	
	if (PosCount > 0)
	{
		Pos = Pos.substr(0, Pos.length() - 1);
	}
	return Pos;
}


bool isPaired (string & GenomePos1, string & GenomePos2, int readLength) //*** with some changes
//----------------------------------------------------------------------
{
	if (GenomePos1 == "" || GenomePos2 == "")
		return false;
	
	rLen = readLength;
	
	string p1, p2, tp;
	string GP1 = "", GP2 = "";
	long n1, n2;
	int cn1, cn2;
	int f1, f2, l1, l2;
	bool rc1, rc2;
	
	bool there_is_num1 = true;
	bool there_is_num2 = true;
	bool flag = false;		
			
	int i = -1, j;
	int i_dot;
	
	while (there_is_num1)
	{
		f1 = i + 1;
		i = GenomePos1.find (",", i + 1);
		
		if (i == string::npos)
		{
			there_is_num1 = false;
			l1 = GenomePos1.length();
		}
		else
		{
			l1 = i - f1;
		}
			
		rc1 = false;		
		p1 = GenomePos1.substr (f1, l1);
		tp = p1;
		
		if (p1.substr (0,1) == "r")
		{
			tp = p1.substr (1, p1.length() - 1);
			rc1 = true;
		}
///////////////  CHANGE  //////////////////
		 
		int i_dot = tp.find (".");	
		cn1 = stoi (tp.substr (0, i_dot));
		n1 = stol (tp.substr(i_dot + 1, tp.length() - i_dot));	
		
///////////////  CHANGE  //////////////////
		
		there_is_num2 = true;
		j = -1;
		
		while (there_is_num2)
		{
			f2 = j + 1;
			j = GenomePos2.find (",", j + 1);
		
			if (j == string::npos)
			{
				there_is_num2 = false;
				l2 = GenomePos2.length();
			}
			else
			{
				l2 = j - f2;
			}
			
			rc2 = false;
			p2 = GenomePos2.substr (f2, l2);
			tp = p2;
			
			if (p2.substr (0,1) == "r")
			{
				tp = p2.substr (1, p2.length() - 1);
				rc2 = true;
			}
///////////////  CHANGE  //////////////////

			int i_dot = tp.find (".");	
			cn2 = stoi (tp.substr (0, i_dot));
			n2 = stol (tp.substr(i_dot + 1, tp.length() - i_dot));	
			/**
			if (cn2 != cn1)
			{
				cout << p1 << "cn1: " << cn1 << " n1: " << n1 << "\n";		
				cout << p2 << "cn2: " << cn2 << " n2: " << n2 << "\n";
				//char ch;
				//cin >> ch;
			}
			*/

///////////////  CHANGE  //////////////////	
						
			int num1 = n1, num2 = n2;
			if (n2 > n1)
			{
				num1 = n2;
				num2 = n1;
			}
			
			//int d = dist -rLen + 1; // --------------- new formula
		
			if  ((cn1 == cn2) &&
			     (num1 - num2 + rLen) <= (dist + sd) && 
			     (num1 - num2 + rLen) >= (dist - sd) &&
			     ((rc1 && !rc2 ) || (!rc1 && rc2)))
			{
				GP1 += p1 + ",";
				GP2 += p2 + ",";
				flag = true;
			}
			
		}
	}
	
	if (flag)
	{
		GenomePos1 = GP1.substr(0, GP1.length() -1);
		GenomePos2 = GP2.substr(0, GP2.length() -1); 
	}
	
	return flag;
}



int NucScore(char Nuc)
//--------------------
{
	switch (Nuc) 
	{
		case 'A':
		case 'a': 
			return A; 
		case 'C':
		case 'c': 
			return C;
		case 'G':
		case 'g': 
			return G;
		case 'T':
		case 't': 
			return T;
		default :
			return 0;
	}
}

int NucScore2(char Nuc)
//---------------------
{
	switch (Nuc) 
	{
		case 'A':
		case 'a': 
			return A2; 
		case 'C':
		case 'c': 
			return C2;
		case 'G':
		case 'g': 
			return G2;
		case 'T':
		case 't': 
			return T2;
		default :
			return 0;
	}
}


inline string trim (const string &t)
//----------------------------------
{
	string str = t;
	size_t found;
	
	found = str.find_last_not_of(WHITESPACE);
	
	if (found != string::npos)
		str.erase(found + 1);
	else
		str.clear();            // str is all whitespace

    return str;
}

string reverseComplement (string read)
{
	string readRC = "";
	int readLength = read.length(); 
	for (int i = readLength - 1; i >= 0; i--)
	{
		switch (read[i]) 
		{
			case 'A': 
				readRC += "T";
				break;
			case 'C': 
				readRC += "G";
				break;
			case 'G': 
				readRC += "C";
				break;
			case 'T': 
				readRC += "A";
				break;
			case 'a': 
				readRC += "t";
				break;
			case 'c': 
				readRC += "g";
				break;
			case 'g': 
				readRC += "c";
				break;
			case 't': 
				readRC += "a";
				break;
			default:
				readRC += "?";
				break;
		}
		
	}
	
	return readRC;
}

string reverse (string str)
{
	string rstr = "";
	int strLength = str.length();
	 
	for (int i = strLength - 1; i >= 0; i--)
	{
		rstr += str[i];	
	}
	
	return rstr;
}

int countUniquePos (string &pos)  //*** without any changes
//------------------------------
{
	string S[50];
	string temp = pos;
	string t;
	int i, unqNum, n = 0;

	while ((i = temp.find (",")) != string::npos)
	{
		S[n++] = temp.substr (0, i);
		temp = temp.substr (i + 1);
	}
	S[n++] = temp;

	i = 0;
	//***? print pos sorting
	unqNum = 0;
	while (i < n)
	{
		t = S[i];
		while ((++i) < n && S[i].compare (t) == 0);
		unqNum++;
	}
	
	if (unqNum == 1)
	{
		pos = t;
	}
	return unqNum;
}

void initBF (int f[], bool isFirstRead)
{
	for (int j = 0; j < 16; j++)
	{
		f[j] = 0;
	}
	f[0] = 1;  //hex: 0x0001 the read is paired in sequencing,
			 //no matter whether it is mapped in a pair
	f[2] = 1; // The query sequence itself is unmapped 
	f[3] = 1; // the mate is unmapped

	if (isFirstRead)				
		f[6] = 1; // the read is the first read in a pair
	else
		f[7] = 1; // the read is the second read in a pair
}

char complement (char nuc)
{
	switch (nuc) 
	{
		case 'A': 
			return 'T';
		case 'C': 
			return 'G';
		case 'G': 
			return 'C';
		case 'T': 
			return 'A';
		case 'a': 
			return 'a';
		case 'c': 
			return 'c';
		case 'g': 
			return 'g';
		case 't': 
			return 't';
		default:
			return '?';
	}
} 

