#include <iostream>
#include <fstream>
#include <string>
#include <stdlib.h>
#include <ctime>
#include <math.h>

 using namespace std;

const int MAX_NUM = 1000;
 
int main(int argc, char *argv[])
{
	// Check the number of parameters
	if (argc < 5) 
	{
	   // Tell the user how to run the program
	   cerr << "Usage: " << argv[0] <<" "
		   << "GenomeFile IndexFile Threshold Rev" 
		   << std::endl;
	   // "Usage messages" are a conventional way of telling the user
	   // how to run a program if they enter the command incorrectly.
	    
	   return 1;
	}
	
	char *GenomeFile = argv[1];
	char *IndexFile = argv[2];
	int Threshold =  atoi(argv[3]);
	int Rev =  atoi(argv[4]);


	int ReadLength = 0;

	string rcc = "rcc.txt";
	ifstream infRL(rcc);
	
	if (!infRL)
	{
		cerr << rcc <<" could not be opened for writing!" << endl;
		return 1;
	}
	
	infRL >> ReadLength;

	infRL.close();

	

/*	
// GenomeFile:EcoliGenome.fa ShortReadLength:45 Kiterate:3

	int kitrate = 3;
	
	int A = 3;
	int C = 20;
	int G = 61;
	int T = 98;
	
	int A2 = 9;
	int C2 = 400;
	int G2 = 3721;
	int T2 = 9604;
*/	

/*
// The best numbers for Ecoli with read length of 101
// one collision
// GenomeFile:EcoliGenome.fa ShortReadLength:101 Kiterate:2

	int kitrate = 2;
	
	int A = 1;
	int C = 13;
	int G = 26;
	int T = 39;
	
	int A2 = 26;
	int C2 = 1;
	int G2 = 39;
	int T2 = 13;
*/	
// GenomeFile:EcoliGenome.fa ShortReadLength:151 Kiterate:2

	int kitrate = 2;
	
	int A = 1;
	int C = 2;
	int G = 5;
	int T = 7;
	
	int A2 = 5;
	int C2 = 1;
	int G2 = 7;
	int T2 = 2;
	
	ifstream inf(GenomeFile);

	// If we couldn't open the input file stream for reading
	if (!inf)
	{
		// Print an error and exit
		cerr << argv[1] <<" could not be opened for reading!" << endl;
		return 1;
	}
	
	// current date/time based on current system
	time_t now = time(0);

	// convert now to string form
	char* dt = ctime(&now);
	cout << "Started at " << dt << "\n";
	
     	int ContigLength [MAX_NUM] = {0};
	string Contig [MAX_NUM] = {""};
	string ContigName [MAX_NUM] = {""};
	string line;
	
	int n = 0; 
	getline(inf, line);
 
	if (line[0] == '>')  // for the first contig
	{
		ContigName[n] = line;
	}
	
	while (!inf.eof())
	{
		getline(inf, line);

		if (line.length() == 0) // for the last empty line
			break;

		if (line[0] == '>')
		{
			ContigLength[n] = Contig[n].length();
		
			ContigName[++n] = line;
		}
		else
		{
			Contig[n] += line;
		}		
	}
	ContigLength[n] = Contig[n].length(); // for the last contig

	int contigsNum = n + 1;  // +1 for intitializing with -1;

	ofstream outf(IndexFile);
	
	if (!outf)
	{
		cerr << argv[2] <<" could not be opened for writing!" << endl;
		return 1;
	}

	string CtgNamesFile(IndexFile);
	CtgNamesFile.append("_name");

	ofstream outf2(CtgNamesFile.c_str());
		
	if (!outf2)
	{
		cerr << CtgNamesFile <<" could not be opened for writing!" << endl;
		return 1;
	}

	
	string Word;
	long Score;
	long Score2;
	int x;
	bool flag;
	
	for (int k = 0; k < contigsNum; k++)
	{
		string CName; 
		int i_space = ContigName[k].find (" ");
		if (i_space != string::npos)
		{
			CName = ContigName[k].substr (1, i_space - 1);	
		}
		else
		{
			CName = ContigName[k].substr (1);
		}

		outf2 << CName << "\t" << ContigLength[k] << "\n";

		for (int i = 0; i <= ContigLength[k] - ReadLength; i++) 
		{
			Word = Contig[k].substr(i, ReadLength);
			Score = 0;
			Score2 = 0;
			flag = true;
			for (int j = Threshold; j < ReadLength - Threshold && flag; j++)
			{
				x = pow ((j + 1), kitrate);
		
				switch (Word[j]) 
				{
					case 'A': 
					case 'a':
						Score += (x * A);
						Score2 += (x * A2); 
						break;
					case 'C': 
					case 'c':
						Score += (x * C);
						Score2 += (x * C2); 
						break;
					case 'G': 
					case 'g':
						Score += (x * G);
						Score2 += (x * G2); 
						break;
					case 'T': 
					case 't':
						Score += (x * T);
						Score2 += (x * T2); 
						break;
					default:
						flag = false;
				}
		
			}
			if (flag)
			{
				if(Rev == 1)
				{
					int P = ContigLength[k] - ReadLength - i;
					outf << "r" << k << "." << P + 1  << "\t" << Score << "." << Score2 << "\n";
				}
				else
				{
					outf << k << "." << i + 1 << "\t" << Score << "." << Score2 << "\n";
				}
			} 	
		}
	}
	
	now = time(0);
	dt = ctime(&now);
	cout << "Finished at " << dt << "\n";
	return 0;
}
