#include <iostream>
#include <fstream>
#include <string>
#include <unordered_map>
#include <stdlib.h>
#include <ctime>

using namespace std;

void makeConsensus (char*, char*);
void getReadProperties (string, string &, long &, string&);
int nucCode(char);
char maxNuc (int, int, int, int);

const int MAX_NUM = 1000;

int main(int argc, char *argv[])
{
	if (argc < 3) 
	{
		cerr << "Usage: " << argv[0] <<" "
		     << "samFile consensusFile\n";    
		return 1;
	}
	
	char *SamFile = argv[1];
	char *ConFile = argv[2];
	
	time_t now = time(0);
	char* dt = ctime(&now);
	cout << "Started at " << dt << "\n";
	
	makeConsensus (SamFile, ConFile);
	
	now = time(0);
	dt = ctime(&now);
	cout << "Finished at " << dt << "\n";
	
	return 0;
}

void makeConsensus (char* SamFile, char* ConFile)
{
	ifstream inSamf(SamFile);
	if (!inSamf)
	{
		cerr << SamFile <<" could not be opened for reading!\n";
		return;
	}
	////////////////////////////////////////

	///////////////////////////////
	unordered_map<string, int> genomeSeqs; 
	///////////////////////////////
	string line;
	string SN [MAX_NUM];
	long LN [MAX_NUM];
	int startSN, startLN;
	long genomeLen;
	int seqNum = 0;
	
	getline (inSamf, line);
	
	while (!inSamf.eof() && line[0] == '@')
	{
		if (line[1] == 'S' && line[2] == 'Q')
		{
			startSN = line.find ("SN");
			startLN = line.find ("LN");
			SN [seqNum] = line.substr (startSN + 3, startLN - startSN - 4);
			LN [seqNum] = stol (line.substr (startLN + 3));

			//-------------------------------
			//cout << seqNum << SN[seqNum] << LN[seqNum] << "\n";
			
			genomeSeqs [SN[seqNum]] = seqNum;
			seqNum++;
		}
		
		getline (inSamf, line);
	}
	///////////////////////////////////////
	
	int i, j, k;

	int *** fr = new int ** [seqNum];
	
	for (k = 0; k < seqNum; k++)
	{
		fr[k] = new int * [4];

		for (i = 0; i < 4; i++)
		{
			fr[k][i] = new int [LN[k] + 1]; //------------- start point for Genome is 1
		}
	
		for (i = 0; i < 4; i++)
		{
			for (j = 1; j < LN[k] + 1; j++) //------------- start point for Genome is 1
			{
				fr[k][i][j] = 0;
			}
		}	
	}

	string read;
	string rName;
	long readPos;
	int readLen;
	int kk;

	while (!inSamf.eof())
	{
		getReadProperties (line, read, readPos, rName);
		
 		if (genomeSeqs.find(rName) != genomeSeqs.end())
		{
			kk = genomeSeqs[rName];

			for (i = 0; i < read.length(); i++)
			{
				if (nucCode(read[i]) != -1)
					fr[kk][nucCode(read[i])][readPos + i]++;
			}
		}
		
		getline (inSamf, line);
		//cout << line << endl; //------------- debuging
	}
	
	
	ofstream outf(ConFile);
	if (!outf)
	{
		cerr << ConFile <<" could not be opened for writing!\n";
		return;
	}
	
	string fileName = SamFile;
	int len = fileName.find (".sam");
	fileName = fileName.substr(0, len);
	
	int nonZero = 0;

	for (k = 0; k < seqNum; k++)	
	{
		outf << ">" << SN [k] << "\n";

		for (i = 1; i < LN[k] + 1; i++) //------------- start point for Genome 0 or 1
		{
			if (fr[k][0][i] > 0) nonZero++;
			if (fr[k][1][i] > 0) nonZero++;
			if (fr[k][2][i] > 0) nonZero++;
			if (fr[k][3][i] > 0) nonZero++;
		
			char mNuc = maxNuc (fr[k][0][i], fr[k][1][i], fr[k][2][i], fr[k][3][i]);
			outf << mNuc;
				
			nonZero = 0;

			//cout << mNuc << endl; //------------- debuging
		}
		if (k < seqNum - 1)
		{
			outf << "\n";
		}
	}
	
}

void getReadProperties (string line, string &read, long &readPos, string& rName)
{
	int k;
	k =-1;
	while (line[++k] != '\t');
	while (line[++k] != '\t');

	rName = "";
	while (line[++k] != '\t')
	{
		rName += line[k];
	}
	//=======================
	
	string pos = "";
	while (line[++k] != '\t')
	{
		pos += line[k];
	}
	
	//cout << pos <<  " " ; //--------debuging

	readPos = atol (pos.c_str());
	
	//=======================
	
	while (line[++k] != '\t');
	while (line[++k] != '\t');
	while (line[++k] != '\t');
	while (line[++k] != '\t');
	while (line[++k] != '\t');
	
	read = "";
	while (line[++k] != '\t')
	{
		read += line[k];
	}
	//=======================
	
	//cout << read[0] << endl; //--------debuging
}

int nucCode(char Nuc)
//---------------------
{
	switch (Nuc) 
	{
		case 'A':
		case 'a': 
			return 0; 
		case 'C':
		case 'c': 
			return 1;
		case 'G':
		case 'g': 
			return 2;
		case 'T':
		case 't': 
			return 3;
		default :
			return -1;
	}
}

char maxNuc (int a, int c, int g, int t)
{
	int m = 0;
	int r = 0;
	char nuc = 'N';
	
	if (a > m)
	{
		m = a;
		nuc = 'A';
	}
	
	if (c > m)
	{
		m = c;
		nuc = 'C';
	}
	else 
	{
		if (c == m && m != 0)
		{
			nuc = 'N';
		}
	}
	
	if (g > m)
	{
		m = g;
		nuc = 'G';
	}
	else 
	{
		if (g == m && m != 0)
		{
			nuc = 'N';
		}	
	}
	
	if (t > m)
	{
		m = t;
		nuc = 'T';
	}
	else 
	{
		if (t == m &&  nuc != 'N')
		{
			nuc = 'N';	
		}
	}
	
	return nuc;
}






