#include <iostream>
#include <fstream>
#include <string>
#include <stdlib.h>
#include <ctime>
#include <math.h>

using namespace std;

const int MAX_NUM = 1000;
const string WHITESPACE = " \t\f\v\n\r";

inline string endTrim (const string &);
 
int main(int argc, char *argv[])
{
	if (argc < 4) 
	{
	   cerr << "Usage: " << argv[0] <<" "
		   << "GenomeFile GenomeFile2 supGenome" 
		   << std::endl;
	   return 1;
	}
	
	char *GenomeFile = argv[1]; //consensus step 1
	char *GenomeFile2 = argv[2]; // reference genome
	char *supGenome = argv[3];
	int Threshold;

	string rcc = "rcc.txt";
	ifstream infRL(rcc);
	
	if (!infRL)
	{
		cerr << rcc <<" could not be opened for writing!" << endl;
		return 1;
	}
	
	infRL >> Threshold;

	infRL.close();
	
	
	ifstream inf(GenomeFile);
	if (!inf)
	{
		cerr << argv[1] <<" could not be opened for reading!" << endl;
		return 1;
	}
	
	
	time_t now = time(0);
	char* dt = ctime(&now);
	cout << "Started at " << dt << "\n";
	
	int ContigLength [MAX_NUM] = {0};
	string Genome [MAX_NUM] = {""};
	string Genome2 [MAX_NUM] = {""};
	string ContigName [MAX_NUM] = {""};

	string line;

///////////////////////////////////

	int n = 0; 
	getline(inf, line);
 
	if (line[0] == '>')  // for the first contig
	{
		ContigName[n] = endTrim (line);
	}
	
	while (!inf.eof())
	{
		getline(inf, line);

		if (line.length() == 0) // for the last empty line
			break;

		if (line[0] == '>')
		{
			ContigLength[n] = Genome[n].length();
			ContigName[++n] = endTrim (line);
		}
		else
		{
			Genome[n] += line;
		}		
	}
	ContigLength[n] = Genome[n].length(); // for the last contig

	int contigsNum = n + 1;

///////////////////////////////////

	
	/////////////////////////////////

	ofstream outf(supGenome);
	if (!outf)
	{
		cerr << argv[3] <<" could not be opened for writing!" << endl;
		return 1;
	}
	
	/////////////////////////////////

	ifstream inf2(GenomeFile2);
	if (!inf2)
	{
		cerr << argv[2] <<" could not be opened for reading!" << endl;
		return 1;
	}

///////////////////////////////////

	n = -1; 
	while (!inf2.eof())
	{
		getline(inf2, line);

		if (line.length() == 0) // for the last empty line
			break;

		if (line[0] != '>')
		{
			Genome2[n] += line;
		}
		else
		{
			n++;
		}		
	}
	

///////////////////////////////////
	

	int i, j, k;
	string contig;
	int startPos, endPos;
	int left, right;

	for (k = 0; k < contigsNum; k++)
	{
		startPos = endPos = 0;
		i = 0;

		while (i < ContigLength[k]) 
		{
			if (Genome[k][i] == 'n' || Genome[k][i] == 'N')  
			{
				startPos = i > Threshold ? i - Threshold : 0;
				endPos = i;
			
				contig = "";
			
				for (j = i - Threshold > 0 ? i - Threshold : 0; j < i; j++)
				{
					contig += Genome2[k][j];
				}
			
	
				while (Genome[k][i] == 'n' || Genome[k][i] == 'N')
				{
					contig += Genome2[k][i];
					i++;
				}
				endPos += i - j - 1;
		
				if (ContigLength[k] >= endPos + Threshold) 
				{
					for (j = i; j < i + Threshold; j++)
					{
						contig += Genome2[k][j];
					}
					endPos += Threshold;
				}
				
				
				outf << ContigName[k] << ":" << startPos + 1 << "-" << endPos + 1 << "\n";
				outf << contig << "\n";
			
			}
			else
			{
				i++;
			}
		
		}

		
	}
	
	outf.close();
	
	now = time(0);
	dt = ctime(&now);
	cout << "Finished at " << dt << "\n";
	return 0;
}

inline string endTrim (const string &t)
{
	string str = t;
	size_t found;
	
	found = str.find_last_not_of(WHITESPACE);
	
	if (found != string::npos)
		str.erase(found + 1);
	else
		str.clear();            // str is all whitespace

    return str;
}
