#include <iostream>
#include <fstream>
#include <string>
#include <stdlib.h>
#include <ctime>
#include <math.h>

 using namespace std;
 
int main(int argc, char *argv[])
{
	// Check the number of parameters
	if (argc < 6) 
	{
	   // Tell the user how to run the program
	   cerr << "Usage: " << argv[0] <<" "
		   << "GenomeFile GenomeFile2 readsfile1 readsfile2 resulltFolder" 
		   << std::endl;
	   // "Usage messages" are a conventional way of telling the user
	   // how to run a program if they enter the command incorrectly.
	    
	   return 1;
	}
	
	char *GenomeFile = argv[1];
	char *CtgFile = argv[2];
	char *readsFile1 = argv[3];
	char *readsFile2 = argv[4];
	char *resultFolder = argv[5];
	
	ifstream inf(GenomeFile);

	if (!inf)
	{
		// Print an error and exit
		cerr << argv[1] <<" could not be opened for reading!" << endl;
		return 1;
	}

	ofstream outf(CtgFile);
	
	if (!outf)
	{
		cerr << argv[2] <<" could not be opened for writing!" << endl;
		return 1;
	}
	
	// current date/time based on current system
	time_t now = time(0);

	// convert now to string form
	char* dt = ctime(&now);
	//cout << "Started at " << dt << "\n";
	
     	
	string contig = "";
	string line;
	
	int n = 0; 
	getline(inf, line);
 
	if (line[0] == '>')  // for the first contig
	{
		outf << ">C" << n + 1 << "***" << line.substr (1) << endl;
	}
	
	while (!inf.eof())
	{
		getline(inf, line);

		if (line.length() == 0) // for the last empty line
			break;

		if (line[0] == '>')
		{
			outf << contig << endl;
			contig = "";
			n++;
			outf << ">C" << n + 1 << "***" << line.substr (1) << endl;
			
		}
		else
		{
			contig += line;
		}		
	}
	
	outf << contig << endl;

	////////////////////////////////////////////

	ifstream inf2(readsFile1);

	if (!inf2)
	{
		cerr << argv[3] <<" could not be opened for reading!" << endl;
		return 1;
	}
	
	string rcc = "rcc.txt";
	//rcc.append (resultFolder);
	//rcc += "/rcc.txt";

	ofstream outf2(rcc);
	
	if (!outf2)
	{
		cerr << "rcc.txt" <<" could not be opened for writing!" << endl;
		return 1;
	}

	double avg = 0;
	int readsNum = 0;

	while(!inf2.eof()) 
	{
		// LINE1: TITLE OF SHORT READ
		getline(inf2, line);
	
		// LINE2: CONTENT OF SHORT READ
		getline(inf2, line);
		
		avg += line.length();
			
		// LINE3: DESCRIPTION OF SHORT READ
		getline(inf2, line);
	
		// LINE4: QUALITY OF SHORT READ
		getline(inf2, line);
		
		readsNum++;
	}

	cout << avg << " " << readsNum << " ";
	avg =  (int) (avg /(readsNum - 1));

	cout << avg << " " << n+1 << " ";

	outf2 << avg << endl;
	outf2 << n + 1 << endl;

	now = time(0);
	dt = ctime(&now);
	//cout << "Finished at " << dt << "\n";
	return 0;
}
